

#include "generalmidi.h"
#include "Trigger.h"
#include "electric_funkatron.h"
#include <SevenSegment.h>
#include "NeckButton.h"
#include "Whammy.h"

// what midi channel we're sending on
// ranges from 1-16 (these are the alesis qs7 channels; they actually correspond to "internal" midi channels 0-15)
int drumchan = 1;


// array to hold all Trigger objects
Trigger triggers[numTriggers];

//--------------------------------------------------------------------------------------------------
// This section deals with the 7-segment display

// using pins 38 through 45
#define A_pin 44
#define B_pin 45
#define C_pin 43
#define D_pin 40
#define E_pin 41
#define F_pin 38
#define G_pin 39
#define RHDP_pin 42

class InvertedMidiChannelSevenSegment: SevenSegment
{
  public:
    void write(int channel);
    InvertedMidiChannelSevenSegment(int anodeA, int anodeB, int anodeC, int anodeD, int anodeE, int anodeF, int anodeG, int RHDP);
};

InvertedMidiChannelSevenSegment::InvertedMidiChannelSevenSegment(int anodeA, int anodeB, int anodeC, int anodeD, int anodeE, int anodeF, int anodeG, int RHDP):
  SevenSegment(anodeD, anodeE, anodeF, anodeA, anodeB, anodeC, anodeG, RHDP)
{
  
}

void InvertedMidiChannelSevenSegment::write(int channel)
{
  if(channel >= 1 && channel <=9)
  {
    SevenSegment::write(channel);
    SevenSegment::unsetDecimalPoint();
  }
  else
  {
    SevenSegment::write(channel % 10);
    SevenSegment::setDecimalPoint();
  }
}

// the actual variable for midi channel display
InvertedMidiChannelSevenSegment midiDisplay(A_pin, B_pin, C_pin, D_pin, E_pin, F_pin, G_pin, RHDP_pin);

//----------------------------------------------------------------------------------------------------
// This section sets up the user-selectable drum kits

/*
  These are preset "kits" that can be user-selected at run-time.
  
  I find it easier to define the kits in separate header files.  See the std_acoustic_kit.h
  file for an example -- each line is a macro of the form
  
  DRUMSOUND(midi_note_value,note_duration,velocity_threshold)
  
  There must be exactly the same number of lines in a kit file as there are triggers!
 */
 // change this based on how many total kits there are
#define numKits 2
NOTE blank_note = { 0, 0, 0 };
NOTE kits[numKits][numTriggers+1] =
{
#define DRUMSOUND(VAL,TIME,VELO) { VAL, TIME, VELO },
   {
// the next line includes all the sound definitions in the kit file
#include "std_acoustic_kit.h"
      blank_note // this is needed to "eat" the final comma.  It is ignored by the rest of the code.
   }
   ,
   {
#include "all_toms_kit.h"
      blank_note
   }
/*
  // sample for adding a third kit:
  , // this comma is important!
  {
#include "my_funky_kit.h"
    blank_note
  }
*/
#undef DRUMSOUND
};


//------------------------------------------------------------------------------------------------------
// This section configures the red neck button for cycling through the midi channel

class MidiChannelChangeButton: public NeckButton 
{
  protected:
    void onPress();
  
  public:
   MidiChannelChangeButton(int); 
};

MidiChannelChangeButton::MidiChannelChangeButton(int pin): NeckButton(pin) { }

void MidiChannelChangeButton::onPress()
{
  drumchan += 1;
  if(drumchan > 16) drumchan = 1;
  setMidiChannel(drumchan);
  Serial.println("midi channel " + drumchan);
}

MidiChannelChangeButton midiChangeButton = MidiChannelChangeButton(red_button);

//--------------------------------------------------------------------------------------------------------
// This section configures the orange neck button to send an "all notes off" -- basically to reset the system
//   in case some things get locked up

class AllNotesOffButton: public NeckButton
{
  public:
    AllNotesOffButton(int pin);
    
  protected:
    void onPress();
};

AllNotesOffButton::AllNotesOffButton(int pin): NeckButton(pin) { }

void AllNotesOffButton::onPress()
{
  for(int i=0; i<numTriggers; i++)
  {
    triggers[i].sendNoteOff();
  }
}

AllNotesOffButton allNotesOffButton = AllNotesOffButton(orange_button);

//------------------------------------------------------------------------------------------------------
// This section is general utilities, variable declarations, etc.



void setMidiChannel(int chan)
{
  // bad input, ignore
  if(chan < 1 || chan > 16) return;
  
  drumchan = chan;
  
  midiDisplay.write(drumchan);
}

//--------------------------------------------------------------------------------------------------------
// whammy-bar stuff

#define whammy_pin A0

WhammyScratcher whammy = WhammyScratcher(whammy_pin);



//--------------------------------------------------------------------------------------------------------
// Standard Arduino setup() and loop() section

void setup() 
{
  // default midi channel
  setMidiChannel(1);
  
  // triggers
  int kit = 0;
  triggers[0] = Trigger(triggerA, kits[kit][0]);
  triggers[1] = Trigger(triggerB, kits[kit][1]);
  triggers[2] = Trigger(triggerC, kits[kit][2]); 
  triggers[3] = Trigger(triggerD, kits[kit][3]);
  triggers[4] = Trigger(triggerE, kits[kit][4]);
  triggers[5] = Trigger(triggerF, kits[kit][5]);
  triggers[6] = Trigger(triggerG, kits[kit][6]);
  triggers[7] = Trigger(triggerH, kits[kit][7]);
  triggers[8] = Trigger(triggerI, kits[kit][8]);
  triggers[9] = Trigger(triggerJ, kits[kit][9]);
  triggers[10] = Trigger(triggerL, kits[kit][10]);
  triggers[11] = Trigger(triggerM, kits[kit][11]);
  //pinMode(ledPin, OUTPUT);
  
  
  // MIDI In wired to TX1 (Serial1) on the MEGA
  Serial1.begin(31250);   // set MIDI baud rate
  
  // regular Serial (Tx) being used for debugging.
#if USE_DEBUG
  Serial.begin(9600);
  Serial.print("Can you hear me now?");
#endif
}

void loop() 
{
  // check for midi channel change
  midiChangeButton.checkForEvent();
  
  // check for emergency "all notes off"
  allNotesOffButton.checkForEvent();
  
  // check for whammy
  whammy.process();
  
  // loop over all triggers
  for(int i=0; i<numTriggers; i++)
  {
    triggers[i].process();
  }
  //triggers[11].process();
}
